/* Internal logging functions. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
 
#include "logfunc.h"
#include "options.h"
#include "modulefunc.h"
#include "version.h"

/* The current log file. All errors are sent to this file. */
char *logfile = NULL;

char *errmsg = NULL;

/* Logs a message. msg is the message. urgency is the urgency. 
   
   The correct usage of this function is:

   asprintf(&errmsg,"some message");
   logmsg(errmsg,urgency);
*/
void logmsg(char *msg,int urgency){
  FILE *logfile_fd;

  /* Is logfile null? */
  if (!logfile)
    logfile = mktempname();

  /* Open up logfile and make a report. */
  logfile_fd = fopen(logfile,"a");
  chmod(logfile,S_IREAD | S_IWRITE);

  fprintf(logfile_fd,"MONITOR Internal Error\n");
  fprintf(logfile_fd,"BEGIN %s\n",monitorversion);
  fprintf(logfile_fd,"URGENCY %d\n",urgency);
  fprintf(logfile_fd,"REPORT\n");
  fprintf(logfile_fd,"%s\n",msg);
  fprintf(logfile_fd,"DONE\n");
  fprintf(logfile_fd,"DONEMONITOR\n");

  fclose(logfile_fd);

#ifdef HAVE_SYSLOG_H
  /* Also log to the syslog. */
  openlog("esmd",LOG_PID,LOG_DAEMON);
  
  switch (urgency) {
  case ESM_LOG_EMERG:
    syslog(LOG_EMERG,"%s",msg);
    break;
  case ESM_LOG_ALERT:
    syslog(LOG_ALERT,"%s",msg);
    break;
  case ESM_LOG_CRIT:
    syslog(LOG_CRIT,"%s",msg);
    break;
  case ESM_LOG_ERR:
    syslog(ESM_LOG_ERR,"%s",msg);
    break;
  case ESM_LOG_WARNING:
    syslog(ESM_LOG_WARNING,"%s",msg);
    break;
  case ESM_LOG_NOTICE:
    syslog(ESM_LOG_NOTICE,"%s",msg);
    break;
  case ESM_LOG_INFO:
    syslog(ESM_LOG_INFO,"%s",msg);
    break;
  case ESM_LOG_DEBUG:
    syslog(ESM_LOG_DEBUG,"%s",msg);
    break;
  }
#endif

  free(msg);
}

static char *id="@(#) $Id: logfunc.c,v 4.0 1999/10/07 22:56:13 pete Exp $";

/*
 * $Log: logfunc.c,v $
 * Revision 4.0  1999/10/07 22:56:13  pete
 * Started work on version 1.0
 *
 * Revision 3.3  1999/09/19 18:25:55  pete
 * Added support for automake
 *
 * Revision 3.2  1999/09/19 16:15:04  pete
 * Added HAVE_SYSLOG_H support.
 *
 * Revision 3.1  1999/09/19 14:26:01  pete
 * Now chmods the internal error log file to 600.
 *
 * Revision 3.0  1999/09/19 01:15:23  pete
 * Started work on 0.03
 *
 * Revision 2.4  1999/09/17 23:08:01  pete
 * Added syslog support.
 *
 * Revision 2.3  1999/09/17 22:50:47  pete
 * Removed the const from const char *msg in the logmsg() prototype.
 *
 * Revision 2.2  1999/09/17 22:35:11  pete
 * Works! syslog logging will be added soon.
 *
 * Revision 2.1  1999/09/17 22:08:47  pete
 * First version.
 *
 */
