/* Monitor functions. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
 
#include "monitor.h"
#include "modulefunc.h"
#include "options.h"
#include "runcmd.h"
#include "version.h"
#include "logfunc.h"

MONITOR *monitorlist;
int nummonitors = 0;

/* Runs all of the monitors once. */
void runmonitors(){
  int i;

  /* Go through the monitor list. Make tmp stdin and stdout files for each command.
     Then run the command. 
  */
  for (i = 0; i < nummonitors; i++){

    if (!daemon_mode)
      printf("Running monitor %s\n",monitorlist[i].title);

    monitorlist[i].stdin_file = mktempname();
    monitorlist[i].stdout_file = mktempname();

    monitorlist[i].pid = re_system(monitorlist[i].cmd,monitorlist[i].stdin_file,monitorlist[i].stdout_file);

    /* Did we get -1? If so log the failure. */
    if (monitorlist[i].pid == -1){
      asprintf(&errmsg,"Couldn't fork while executing %s.\n",monitorlist[i].cmd);
      logmsg(errmsg,ESM_LOG_ERR);
    }
  }

  /* Make all remaining monitors exit. */
  for (i = 0; i < nummonitors; i++){
    wait_module(monitorlist[i].pid,monitortimeout);
  }
}

/* Adds a monitor. */
void addmonitor(MONITOR newmon){
  MONITOR *newspot;

  /* Which monitor list should we add the monitor to? */
  if (!(newmon.flags & MONITOR_FLAG_DAEMON)){
    nummonitors++;
    monitorlist = realloc(monitorlist,sizeof(MONITOR) * nummonitors);
    newspot = &monitorlist[nummonitors - 1];
  }
  else{
    numdmonitors++;
    dmonitorlist = realloc(dmonitorlist,sizeof(MONITOR) * numdmonitors);
    newspot = &dmonitorlist[numdmonitors - 1];
  }

  /* Since the strings are just pointers to a string we have make a
     copy of the strings. Otherwise they might be overwritten later.
  */
  newspot->title = malloc(strlen(newmon.title) + 1);
  newspot->title[0] = '\000';
  newspot->title = strcpy(newspot->title,newmon.title);
  
  newspot->cmd = malloc(strlen(newmon.cmd) + 1);
  newspot->cmd[0] = '\000';
  newspot->cmd = strcpy(newspot->cmd,newmon.cmd);
  
  newspot->flags = newmon.flags;
  newspot->pid = 0;
}

static char *id="@(#) $Id: monitor.c,v 4.0 1999/10/07 22:56:14 pete Exp $";

/* 
 * $Log: monitor.c,v $
 * Revision 4.0  1999/10/07 22:56:14  pete
 * Started work on version 1.0
 *
 * Revision 3.2  1999/09/21 00:55:01  pete
 * Now uses the wait_module function.
 *
 * Revision 3.1  1999/09/19 18:25:56  pete
 * Added support for automake
 *
 * Revision 3.0  1999/09/19 01:15:24  pete
 * Started work on 0.03
 *
 * Revision 2.7  1999/09/17 22:49:20  pete
 * Added support/improved support for logmsg()
 *
 * Revision 2.6  1999/09/17 15:03:00  pete
 * Fixed some buffer overruns. Both string allocation problems and also
 * I was incrementing nummonitors instead of numdmonitors once.
 *
 * Revision 2.5  1999/09/16 22:56:24  pete
 * Removed the dmonitor list and moved it to monitor.c
 *
 * Revision 2.4  1999/09/16 21:28:34  pete
 * Bug fix.
 *
 * Revision 2.3  1999/09/16 20:39:44  pete
 * Added dmonitorlist. Added support for dmonitorlist to addmonitor()
 *
 * Revision 2.2  1999/09/16 20:29:15  pete
 * Now sets the pid of new monitors to 0
 *
 * Revision 2.1  1999/09/16 20:27:30  pete
 * Now copies the flags var in addmonitor()
 *
 * Revision 2.0  1999/09/15 16:05:52  pete
 * Started work on version 0.02
 *
 * Revision 1.18  1999/09/12 19:44:11  pete
 * Fixed a bug where addsender and addmonitor were overwriting the title with
 * the cmd. Also changed some printf's
 *
 * Revision 1.17  1999/09/12 18:52:32  pete
 * *** empty log message ***
 *
 * Revision 1.16  1999/09/11 20:15:56  pete
 * Changed tempname to POSIX complient mktempname
 *
 * Revision 1.15  1999/09/11 02:13:03  pete
 * Removed some debugging printf's
 *
 * Revision 1.14  1999/09/11 02:07:34  pete
 * Added ID and LOG tags after much work. :)
 *
 */
