/* Extensible System Monitor 0.01 

   main.c - Parses the command line options. Starts the main loop.

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "main.h"
#include "loop.h"
#include "version.h"
#include "getopt.h"

int notifylevel = ESM_LOG_EMERG;

int maxmsgs = 2048;

int savepri = ESM_LOG_ERR;

int forcecheck_interval = 1800; /* 30 minutes */

void printhelp();
void printversion();

int main(int argc,char *argv[]){
  int c;

  /* Parse the command line options. */
  while (1)
    {
      int option_index = 0;
      static struct option long_options[] =
      {
	{"notifylevel", 1, 0, 0},
	{"maxmsgs", 1, 0, 0},
	{"savepri", 1, 0, 0},
	{"help", 0, 0, 0},
	{"version", 0, 0, 0},
	{"fcint", 1, 0, 0},
	{0, 0, 0, 0}
      };
      
      c = getopt_long (argc, argv, "n:m:s:hv",
		       long_options, &option_index);
      if (c == -1)
	break;

      switch (c)
	{
	case 0:
	  if (option_index == 0){
	    notifylevel = atoi(optarg);
	  }
	  else if (option_index == 1){
	    maxmsgs = atoi(optarg);
	  }
	  else if (option_index == 2){
	    savepri = atoi(optarg);
	  }
	  else if (option_index == 3){
	    printhelp();
	    exit(EXIT_SUCCESS);
	  }
	  else if (option_index == 4){
	    printversion();
	    exit(EXIT_SUCCESS);
	  }
	  else if (option_index == 5){
	    forcecheck_interval = atoi(optarg);
	  }
	  break;
	  
	case 'n':
	  notifylevel = atoi(optarg);
	  break;

	case 'm':
	  maxmsgs = atoi(optarg);
	  break;

	case 's':
	  savepri = atoi(optarg);
	  break;

	case 'h':
	  printhelp();
   	  exit(EXIT_SUCCESS);
	  break;
       
	case 'v':
	  printversion();
	  exit(EXIT_SUCCESS);
	  break;

	case '?':
                 break;

	default:
                 fprintf (stderr,"?? getopt returned character code 0%o ??\n", c);

	}
    }

  if (optind < argc)
    {
      fprintf (stderr,"non-option ARGV-elements: ");
      while (optind < argc)
	fprintf (stderr,"%s ", argv[optind++]);
      fprintf (stderr,"\n");
    }

  mainloop();

  return 0;
}

void printhelp(){
  printf("Usage: esmsyslog [OPTION]...\n");
  printf("ESMSysLog Daemon Monitor.\n");
  printf("\n");
  printf("  -n x, --notifylevel x    Set the notifylevel to x.\n");
  printf("  -m x, --maxmsgs x        Set maxmsgs to x.\n");
  printf("  -s x, --savepri x        Set the save priority to x.\n");
  printf("  --fcint x                Set the force check interval to x.\n");
  printf("  -h, --help               Displays this screen.\n");
  printf("  -v, --version            Displays version information.\n");
  printf("\n");
  printf("Report bugs to pc_toilet@hotmail.com\n");
  exit(EXIT_SUCCESS);
}

void printversion(){
  printf("ESMSysLog %s\n",version);
  printf("Copyright (C) 1999 Peter Todd\n");
  printf("\n");
  printf("esmsyslog comes with NO WARRANTY,\n");
  printf("to the extent permitted by law.\n");
  printf("You may redistribute copies of esmd\n");
  printf("under the terms of the GNU General Public License.\n");
  printf("For more information about these matters,\n");
  printf("see the files named COPYING.\n");
  exit(EXIT_SUCCESS);
}

static char *id="@(#) $Id: main.c,v 2.0 1999/10/07 23:26:50 pete Exp $";

/* 
 * $Log: main.c,v $
 * Revision 2.0  1999/10/07 23:26:50  pete
 * Started work on version 1.1
 *
 * Revision 1.10  1999/09/21 21:34:46  pete
 * Added forcecheck_interval var and --fcint option.
 *
 * Revision 1.9  1999/09/21 17:31:12  pete
 * Removed a debug message.
 *
 * Revision 1.8  1999/09/21 17:07:09  pete
 * Added cmd line options.
 *
 * Revision 1.7  1999/09/21 16:37:59  pete
 * Removed msgheadroom var.
 *
 * Revision 1.6  1999/09/21 01:15:23  pete
 * Added savepri var.
 *
 * Revision 1.5  1999/09/21 01:08:53  pete
 * Added msgheadroom var.
 *
 * Revision 1.4  1999/09/21 01:05:09  pete
 * Added notifylevel and maxmsgs vars.
 *
 * Revision 1.3  1999/09/20 14:23:34  pete
 * Moved most of the code to loop.c
 *
 * Revision 1.2  1999/09/20 01:22:22  pete
 * Made fopens for each FIFO.
 *
 * Revision 1.1  1999/09/20 00:48:08  pete
 * First version.
 *
 */
